@@ -7,14 +7,11 @@ set -ev
 # Explicitly disable sparse support for llvm because some travis
 # environments claim to have LLVM (llvm-config exists and works) but
 # linking against it fails.
+# Disabling sqlite support because sindex build fails and we don't
+# really need this utility being installed.
 git clone git://git.kernel.org/pub/scm/devel/sparse/sparse.git
 cd sparse
-# Commit bb1bf748580d ("cgcc: gendeps for -MM, -MD & -MMD too") makes
-# sparse ignore almost all source files, because 'make' uses '-MD' to
-# generate dependencies as a side effect within compilation commands.
-git revert bb1bf748580d --no-commit
-git diff HEAD
-make -j4 HAVE_LLVM= install
+make -j4 HAVE_LLVM= HAVE_SQLITE= install
 cd ..
 
 pip install --disable-pip-version-check --user six flake8 hacking
